/**
 * \defgroup vector 32bit NCO algorithm implementation
 * \ingroup libbfin32
 * \details
 * A set of 32bit fixed point vector arithmetic functions.
 * There are 2 types of major triadic functions.
 *
 * The first type of functions are "vector op vector -> vector" type.
 * This type of function applies operation for each pair of elements of vectors.
 * That mean, there are no inner product operation in this library.
 *
 * The second type of functions are "scaler op vector -> vector" type.
 * This type of function applies operation between the scalar and for each elements of vector.
 * The function name has "_svv" suffix, which represent "Scalar and Vector to Vector".
 */
/*@{*/


/**
 * \file fr32_vector.h
 * \date 2014/Mar/22
 * \author takemasa
 * \brief Basic vector implementation
 *
 */

#ifndef FX32_VECTOR_H_
#define FX32_VECTOR_H_

#ifdef __i386__
#include "fract_x86.h"
#else
#include <fract.h>
#endif

#include <limits.h>


    /**
     * \brief vector addition A+B => C
     * \param a input vector A
     * \param b input vector B
     * \param c input vector C
     * \param count length of vector
     * \details
     * Add the 32bit fixed point A to B, then, store the result to vector C.
     */
void fr32_vector_add(
        const fract32 a[],
        const fract32 b[],
        fract32 c[],
        int count);

    /**
     * \brief vector subtraction A-B => C
     * \param a input vector A
     * \param b input vector B
     * \param c output vector C
     * \param count length of vector
     * \details
     * Subtract the 32bit fixed point B from A, then, store the result to vector C.
     */
void fr32_vector_sub(
        const fract32 a[],
        const fract32 b[],
        fract32 c[],
        int count);

/**
 * \brief vector addition A+B => C
 * \param a input scalar A
 * \param b input vector B
 * \param c output vector C
 * \param count length of vector
 * \details
 * Add the 32bit fixed point scalar A to vector B, then, store the result to vector C.
 */
void fr32_vector_add_svv(
    const fract32 a,
    const fract32 b[],
    fract32 c[],
    int count);

/**
 * \brief vector Sbutraction A-B => C
 * \param a input scalar A
 * \param b input vector B
 * \param c output vector C
 * \param count length of vector
 * \details
 * Subtract the 32bit fixed point vector B from scalar A , then, store the result to vector C.
 */
void fr32_vector_sub_svv(
    const fract32 a,
    const fract32 b[],
    fract32 c[],
    int count);

    /**
     * \brief vector multiplying A*B => C
     * \param a input vector A
     * \param b input vector B
     * \param c output vector C
     * \param count length of vector
     * \details
     * Multiply the 32bit fixed point A by B, then, store the result to vector C.
     */
void fr32_vector_mul(
        const fract32 a[],
        const fract32 b[],
        fract32 c[],
        int count);

    /**
     * \brief vector multiplying A*B => C
     * \param a input scalar A
     * \param b input vector B
     * \param c output vector C
     * \param count length of vector
     * \details
     * Multiply the 32bit fixed point vector B by scalar A, then, store the result to vector C.
     */
void fr32_vector_mul_svv(
        const fract32 a,
        const fract32 b[],
        fract32 c[],
        int count);

    /**
     * \brief complex vector multiplying A*B => C
     * \param ar input vector re(A)
     * \param ai input vector im(A)
     * \param br input vector re(B)
     * \param bi input vector im(B)
     * \param cr output vector re(C)
     * \param ci output vector im(C)
     * \param count length of vector
     * \details
     * Mulitiply the complex fxied point vector A by B. Then, store the result to vector C
     */
void fr32_vector_complex_mul(
        const fract32 ar[],
        const fract32 ai[],
        const fract32 br[],
        const fract32 bi[],
        fract32 cr[],
        fract32 ci[],
        int count
        );

    /**
     * \brief vector negate -A => B
     * \param a input vector A
     * \param b output vector B
     * \param count length of vector
     * \details
     * Negate the input vector A, then, store the result to vector B.
     */
void fr32_vector_neg(
        const fract32 a[],
        fract32 b[],
        int count);

    /**
     * \brief vector fill A => B
     * \param a input scalar A
     * \param b output vectr B
     * \param count length of vector
     * \details
     * Fill output vector B by input scalar A
     */
void fr32_vector_fill(
        const fract32 a,
        fract32 b[],
        int count);



#endif /* FX32_VECTOR_H_ */


/*@}*/
